EXPORTS
	??0CCommsProviderModule@@IAE@AAVRThread@@@Z @ 1 NONAME ; CCommsProviderModule::CCommsProviderModule(class RThread &)
	??0CRootServer@@IAE@HABVTPolicy@CPolicyServer@@@Z @ 2 NONAME ; CRootServer::CRootServer(int, class CPolicyServer::TPolicy const &)
	??1CCommsProviderModule@@UAE@XZ @ 3 NONAME ; CCommsProviderModule::~CCommsProviderModule(void)
	??1CRootServer@@UAE@XZ @ 4 NONAME ; CRootServer::~CRootServer(void)
	?ConstructL@CCommsProviderModule@@IAEXPAVCRootServer@@ABVTRSStartModuleParams@RootServer@@PAVHBufC8@@@Z @ 5 NONAME ; void CCommsProviderModule::ConstructL(class CRootServer *, class RootServer::TRSStartModuleParams const &, class HBufC8 *)
	?ConstructL@CRootServer@@IAEXABUTRootServerStartupInfo@@@Z @ 6 NONAME ; void CRootServer::ConstructL(struct TRootServerStartupInfo const &)
	?CpmCount@CRootServer@@QAEHXZ @ 7 NONAME ; int CRootServer::CpmCount(void)
	?DisconnectSession@CRootServer@@UAEXPBVCRootServerSession@@@Z @ 8 NONAME ; void CRootServer::DisconnectSession(class CRootServerSession const *)
	?IsCallerConfigurator@CRootServer@@UAEHPBVCRootServerSession@@ABVRMessage2@@@Z @ 9 NONAME ; int CRootServer::IsCallerConfigurator(class CRootServerSession const *, class RMessage2 const &)
	?LazyLoadL@CRootServer@@UAEXXZ @ 10 NONAME ; void CRootServer::LazyLoadL(void)
	?NewSessionL@CRootServer@@UBEPAVCSession2@@ABVTVersion@@ABVRMessage2@@@Z @ 11 NONAME ; class CSession2 * CRootServer::NewSessionL(class TVersion const &, class RMessage2 const &) const
	?ReceivedMessage@CRootServer@@UAEXABV?$TBuf8@$0CA@@@ABVTCFMessage@CommsFW@@@Z @ 12 NONAME ; void CRootServer::ReceivedMessage(class TBuf8<32> const &, class CommsFW::TCFMessage const &)
	?RunL@CCommsProviderModule@@EAEXXZ @ 13 NONAME ; void CCommsProviderModule::RunL(void)
	?RunRootServer@@YAHABUTRootServerStartupInfo@@@Z @ 14 NONAME ; int RunRootServer(struct TRootServerStartupInfo const &)
	?SuddenDeath@CCommsProviderModule@@UAEXH@Z @ 15 NONAME ; void CCommsProviderModule::SuddenDeath(int)
	?SuddenDeath@CRootServer@@QAEXH@Z @ 16 NONAME ; void CRootServer::SuddenDeath(int)
	?Disconnect@CRootServerSession@@MAEXABVRMessage2@@@Z @ 17 NONAME ; void CRootServerSession::Disconnect(class RMessage2 const &)
	??0CRootServerSession@@QAE@PBVCRootServer@@@Z @ 18 NONAME ; CRootServerSession::CRootServerSession(class CRootServer const *)
	??1CRootServerSession@@UAE@XZ @ 19 NONAME ; CRootServerSession::~CRootServerSession(void)
	?ServiceL@CRootServerSession@@UAEXABVRMessage2@@@Z @ 20 NONAME ; void CRootServerSession::ServiceL(class RMessage2 const &)
	?SuddenDeath@CRootServerSession@@UAEXH@Z @ 21 NONAME ; void CRootServerSession::SuddenDeath(int)


